import 'package:ecc_encryption/src/models/bit.dart';
import 'package:eosdart_ecc/eosdart_ecc.dart';
import 'package:encrypt/encrypt.dart' as crypt;

/// ECC Encryption class which provides methods to encrypt and decrypt data also provides additional methods to get public and private keys
class ECC extends Algorithm {
  /// initialise the ECC class with the WIF(Wallet Import Format) private key
  /// [wifPrivateKey] - Private key in WIF format
  ECC.initialise({required String wifPrivateKey}) : super() {
    initialise(wifPrivateKey: wifPrivateKey);
  }
}

/// Algorithm class which holds the algorithm methods
class Algorithm {
  late String _privateKey;
  late String _publicKey;

  /// initialise the ECC class with the WIF(Wallet Import Format) private key
  /// [wifPrivateKey] - Private key in WIF format
  initialise({required String wifPrivateKey}) {
    EOSPrivateKey eosPrivateKey = EOSPrivateKey.fromString(wifPrivateKey);
    EOSPublicKey eosPublicKey = eosPrivateKey.toEOSPublicKey();
    _privateKey = eosPrivateKey.toString();
    _publicKey = eosPublicKey.toString();
  }

  /// Getter for the public key
  String get publicKey => _publicKey;

  /// Getter for the private key
  String get privateKey => _privateKey;

  /// Encrypt the data using the public key and the AES algorithm using AESMode.ecb
  /// [data] - Data to be encrypted
  /// [bitLength] - AES BitLength to be encrypted. Default is AES32
  String? encrypt(
      {required String data, BitLength bitLength = BitLength.aes32}) {
    String? plainText = data;
    if (plainText == '' || plainText == 'null' || plainText == '-') {
      return null;
    } else {
      int bit = 32;
      switch (bitLength) {
        case BitLength.aes16:
          bit = 16;
          break;
        case BitLength.aes24:
          bit = 24;
          break;
        default:
          bit = 32;
          break;
      }
      final key = crypt.Key.fromLength(bit);
      final iv = crypt.IV.fromUtf8(publicKey);
      final encryptor = crypt.Encrypter(
        crypt.AES(
          key,
          mode: crypt.AESMode.ecb,
        ),
      );
      return encryptor.encrypt(plainText, iv: iv).base64;
    }
  }

  /// Decrypt the data using the private key and the AES algorithm using AESMode.ecb
  /// [data] - Data to be decrypted
  /// [bitLength] - AES BitLength to be decrypted. Default is AES32
  String? decrypt(
      {required String data, BitLength bitLength = BitLength.aes32}) {
    String? encrypted = data;
    if (encrypted == '' || encrypted == 'null' || encrypted == '-') {
      return null;
    } else {
      int bit = 32;
      switch (bitLength) {
        case BitLength.aes16:
          bit = 16;
          break;
        case BitLength.aes24:
          bit = 24;
          break;
        default:
          bit = 32;
          break;
      }
      final key = crypt.Key.fromLength(bit);
      final iv = crypt.IV.fromUtf8(privateKey);
      final encryptor = crypt.Encrypter(
        crypt.AES(
          key,
          mode: crypt.AESMode.ecb,
        ),
      );
      return encryptor.decrypt64(encrypted, iv: iv).toString();
    }
  }
}
